/*
Solar Scintillation Seeing Monitor Sketch v1.1

This is a modified version of the sketch described in
https://www.ecu.edu/cs-cas/physics/upload/An-Inexpensive-Solar-Scintillation-Seeing-Monitor-Circuit-with-Arduino-Interface-final2.pdf

Read 'NUM_SAMPLES' of analog input readings on pin 'A1' and 'A0' and find the value of
4.46*RMS(Intensity_A1)/AVE-Intensity_A0. The result is sent to serial monitor or TXRX
output. External Scintillation Monitor circuit gain set to 425.5X. 
*/

#include <LiquidCrystal.h>

#define NUM_SAMPLES 2000 // number of samples used to find RMS value
#define NUM_RUNS 5 // number of samples used to find RMS value

LiquidCrystal lcd(8,9,4,5,6,7);   // setup LCD shield

float longSeeingSum;
float longInputSum;
int longRun;
float longSeeing;


void setup() {
   lcd.begin(16, 2);          // init LCD shield
   Serial.begin(115200);      // innit serial interface
   longSeeingSum = 0;
   longInputSum = 0;
   longSeeing = 0;
   longRun = 0;
}

void loop() {
  int i,sensorValue1,sensorValue2;
  float Value,ValueA,ValueB,IValue,IValueB;
  
  ValueB=0;    // Seeing
  IValue=0;   
  IValueB=0;   // Input level
  
  for ( i=0; i< NUM_SAMPLES; i++)
  {
    sensorValue2 = analogRead(A2); // find the RMS value
    Value=(sensorValue2-511.)*2.5/511.;
    Value=sq(Value);
    ValueB=ValueB+Value;
    sensorValue1 = analogRead(A1); // find the average value of solar intensity
    IValue=(sensorValue1-511.)*2.5/511.;
    IValueB=IValueB+IValue;
  }
  longRun++;
  
  longSeeingSum += ValueB;
  longInputSum += IValueB;
 
  if (longRun >= (NUM_RUNS))
  {
    longInputSum = longInputSum /  (NUM_SAMPLES*NUM_RUNS);
    longSeeingSum = longSeeingSum / (NUM_SAMPLES*NUM_RUNS);
    longSeeingSum = sqrt(longSeeingSum);
    longSeeingSum=4.46*longSeeingSum;
    longSeeingSum=longSeeingSum-(0.05);
    longSeeing=longSeeingSum/longInputSum;
     
    longSeeingSum = 0;
    longInputSum = 0;
    longRun = 0;
  }
  
  
  IValueB=IValueB/NUM_SAMPLES+.0; // + or - small dc off-set
  Serial.print("A0: ");
  Serial.println(IValueB,2);

  lcd.setCursor(0, 1);
  lcd.print("Input:          ");
  lcd.setCursor(7, 1);
  lcd.print(IValueB,2);
  
  ValueB=ValueB/NUM_SAMPLES;
  ValueB=sqrt(ValueB);
  ValueB=4.46*ValueB;
  ValueB=ValueB-.05; // + or - small dc off-set
  ValueB=ValueB/IValueB;
  
  //if (IValueB<0.5) ValueB=0; // Discriminate for clouds  
  //if (ValueB>10.0) ValueB=0; // Discriminate for clouds 
  Serial.print("A1: ");
  Serial.println(ValueB,2);

//  if (ValueB < 0)
//     ValueB *= -1;
  if (ValueB >= 10) {
    ValueB = 9.99;
  }
  if (longSeeing >= 10) {
    longSeeing = 9.99;
  }
  Serial.print("C2: ");
  Serial.println(longSeeing,2);

  lcd.setCursor(0, 0);
  lcd.print("Level:          ");
  lcd.setCursor(7, 0);
  lcd.print(ValueB,2);
  lcd.setCursor(12, 0);
  lcd.print(longSeeing,2);

}
